@@ -279,8 +279,8 @@ fb_initcolors(int fd, int gray)
 #endif
 	break;
     default:
-	fprintf(stderr, "Oops: %i bit/pixel ???\n",
-		fb_var.bits_per_pixel);
+	fb_cleanup();
+	fprintf(stderr, "Oops: %i bit/pixel ???\n", fb_var.bits_per_pixel);
 	exit(1);
     }
 
@@ -316,10 +316,8 @@ tty_cleanup(void)
 static void
 text_init(char *font)
 {
-    char   *fonts[2] = { font, NULL };
-
     if (NULL == f)
-	f = fs_consolefont(font ? fonts : NULL);
+	f = fs_consolefont(font ? font : NULL);
     if (NULL == f) {
 	fprintf(stderr,"no font available\n");
 	exit(1);
@@ -379,6 +377,14 @@ static void do_capture(int from, int to, int tmp_switch)
 	break;
     }
 
+    /* Sanity check: don't accept resolution higher than framebuffer */
+    if (ww && hh) {
+	    if (ww > fb_var.xres)
+		    ww = fb_var.xres;
+	    if (hh > fb_var.yres)
+		    hh = fb_var.yres;
+    }
+
     /* on */
     memset(&buf,0,sizeof(buf));
     switch (to) {
@@ -406,14 +412,16 @@ static void do_capture(int from, int to, int tmp_switch)
 	if (0 != ng_grabber_setformat(&fmt,1)) {
 	    gfmt = fmt;
 	    if (NULL == (conv = ng_grabber_findconv(&gfmt,0))) {
-		fprintf(stderr,"can't fint useful capture format\n");
+		fb_cleanup();
+		fprintf(stderr,"can't find useful capture format\n");
 		exit(1);
 	    }
 	    ch = ng_convert_alloc(conv,&gfmt,&fmt);
 	    ng_convert_init(ch);
 	}
-	dx += (fb_var.xres-24-fmt.width)/2;
-	dy += (fb_var.yres-16-fmt.height)/2;
+
+	dx += (fb_var.xres - 24 - fmt.width)/2;
+	dy += (fb_var.yres - 16 - fmt.height)/2;
 
 	if (f_drv & CAN_CAPTURE)
 	    drv->startvideo(h_drv,-1,2);
@@ -465,6 +473,12 @@ static void do_capture(int from, int to, int tmp_switch)
 	}
 	break;
     }
+
+    /* Sanity checks: some of the formula above could overflow */
+    if (dx < 0 || dx > fb_var.xres)
+	dx = 0;
+    if (dy < 0 || dy > fb_var.yres)
+	dy = 0;
 }
 
 static void
@@ -507,7 +521,9 @@ static void
 do_fullscreen(void)
 {
     do_va_cmd(2,"capture","off");
-    quiet = !quiet;
+    if (quiet < 0)
+	text_init(fontfile);
+    quiet = quiet ? 0 : 1;
     fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
     do_va_cmd(2,"capture","on");
 }
@@ -528,6 +544,7 @@ grabber_init(void)
 
     drv = ng_vid_open(&ng_dev.video, ng_dev.driver, &screen, 0, &h_drv);
     if (NULL == drv) {
+	fb_cleanup();
 	fprintf(stderr,"no grabber device available\n");
 	exit(1);
     }
@@ -582,12 +599,12 @@ scaler_test(int off)
 int
 main(int argc, char *argv[])
 {
-    int             i,key,c,gray=0,rc,vt=0,fps=0,t1,t2,lirc,js,err,mute=1,fdmax;
-    unsigned int    ui;
+    int             i,key,c,help=0,gray=0,rc,vt=0,fps=0,t1,t2,lirc,js,err,mute=1,fdmax, linesize;
+    unsigned long   ui;
     unsigned long   freq;
     struct timeval  tv;
     time_t          t;
-    char            text[80],event[64],*env,*dst;
+    char            text1[145], text2[145], event[64], *env, *src, *dst;
     fd_set          set;
     struct sigaction act,old;
 
@@ -602,7 +619,7 @@ main(int argc, char *argv[])
     ng_init();
     for (;;) {
 	double val;
-	c = getopt(argc, argv, "Mgvqxk2d:o:s:c:f:m:z:t:j:D:");
+	c = getopt(argc, argv, "hMgvqkd:o:s:c:f:m:z:t:j:D:");
 	if (c == -1)
 	    break;
 	switch (c) {
@@ -610,7 +627,7 @@ main(int argc, char *argv[])
 	    if(sscanf(optarg, "%lf", &val) == 1) {
 		if(val < 0.1 || val > 10)
 		    fprintf(stderr, "gamma value is out of range.  must be "
-			    "0.1 < value < 10.0\n");
+			    "0.1 <= value <= 10.0\n");
 		else
 		    fbgamma = 1.0 / val;
 	    }
@@ -635,7 +652,7 @@ main(int argc, char *argv[])
 	    ng_debug++;
 	    break;
 	case 'q':
-	    quiet = 1;
+	    quiet = -1;
 	    break;
 	case 'd':
 	    fbdev = optarg;
@@ -661,13 +678,40 @@ main(int argc, char *argv[])
 	case 'j':
 	    joydev = optarg;
 	    break;
+	case 'h':
+	    help = 1;
+	    break;
 	default:
+	    fprintf(stderr, "Try '%s -h' for more information.\n", argv[0]);
 	    exit(1);
 	}
     }
 
+    if (help) {
+	fprintf(stderr, "Usage:\n\t%s [options]\n\nwhere [options] are:\n\n", argv[0]);
+	fprintf(stderr, "\t-h - print this text\n");
+	fprintf(stderr, "\t-M - enable Matrox mode\n");
+	fprintf(stderr, "\t-g - sets a gray colormap. Works only on 8 bits per pixel modes (256 color mode)\n");
+	fprintf(stderr, "\t-v - increase debug level\n");
+	fprintf(stderr, "\t-q - quiet mode: don't output any text at the framebuffer. Show just the camera stream\n");
+	fprintf(stderr, "\t-k - keep capture enabled when switching consoles\n");
+
+	fprintf(stderr, "\t-d <device> - use <device> as the framebuffer devicen");
+	fprintf(stderr, "\t-o <base> - base string prefix for snapshot file names. Default: snap\n");
+	fprintf(stderr, "\t-s <resolution> - V4L2 capture resolution. The <resolution> is at <width>x<height> format, e. g. 800x600\n");
+	fprintf(stderr, "\t-c <device> - use <device> as video4linux device\n");
+	fprintf(stderr, "\t-f <filename> - use <filename> as the font console file. Not used on quiet mode. Default: lat1-16\n");
+	fprintf(stderr, "\t-m <mode> - framebuffer mode, as defined at /etc/fb.modes\n");
+	fprintf(stderr, "\t-z <gamma> - Sets the Gamma value. It should be between 0.1 and 10.0\n");
+	fprintf(stderr, "\t-t [<vt>] - virtual tty to open\n");
+	fprintf(stderr, "\t-j <device> - use <device> as a joystick to control the capture\n");
+	fprintf(stderr, "\t-D <name> - use <name> as video4linux driver. Default: libv4l\n");
+	exit(1);
+    }
+
     do_overlay = 1;
-    text_init(fontfile);
+    if (!quiet)
+	text_init(fontfile);
     fb = fb_init(fbdev,mode,vt);
     fb_catch_exit_signals();
     fb_initcolors(fb,gray);
@@ -697,7 +741,6 @@ main(int argc, char *argv[])
     exit_hook         = do_exit;
     fullscreen_hook   = do_fullscreen;
 
-    tty_init();
     memset(&act,0,sizeof(act));
     act.sa_handler = ctrlc;
     sigemptyset(&act.sa_mask);
@@ -709,7 +752,7 @@ main(int argc, char *argv[])
     audio_init();
 
     /* build channel list */
-    parse_config();
+    parse_config(1);
     channel_menu();
 
     init_overlay();
@@ -743,29 +786,32 @@ main(int argc, char *argv[])
     }
 #endif
 
+    tty_init();
     fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
     for (;!sig;) {
 	if ((fb_switch_state == FB_ACTIVE || keep_dma_on) && !quiet) {
-	    /* clear first lines */
-	    fb_memset(fb_mem+fb_mem_offset,0,f->height*fb_fix.line_length);
 	    if (message[0] != '\0') {
-		strcpy(text,message);
+		strcpy(text1, message);
 	    } else {
-		sprintf(text,"Framebuffer TV - %s",default_title);
+		sprintf(text1, "Framebuffer TV - %s",default_title);
 	    }
 	    /* debugging + preformance monitoring */
 	    switch (cur_capture) {
 	    case CAPTURE_GRABDISPLAY:
-		sprintf(text+strlen(text), " - grab %d.%d fps",fps/5,(fps*2)%10);
+		sprintf(text1 + strlen(text1), " - grab %d.%d fps",
+			fps / 5, (fps * 2) % 10);
 		break;
 	    }
-	    text_out(0,0,text);
 
-	    if (dy > 0) {
-		/* display time */
-		time(&t);
-		strftime(text,16,"%H:%M",localtime(&t));
-		text_out(fb_var.xres - text_width(text) - f->width, 0, text);
+	    /* display time */
+	    time(&t);
+	    strftime(text2, 16, "%H:%M", localtime(&t));
+
+	    if (dy >= f->height) {
+		/* clear first lines */
+		fb_memset(fb_mem+fb_mem_offset,0,f->height*fb_fix.line_length);
+		text_out(0,0,text1);
+		text_out(fb_var.xres - text_width(text2) - f->width, 0, text2);
 	    }
 	}
 	if (switch_last != fb_switch_state)
@@ -797,18 +843,37 @@ main(int argc, char *argv[])
 		fps++;
 		/* grab + convert frame */
 		if (NULL == (buf = ng_grabber_grab_image(0))) {
+		    do_va_cmd(2,"capture","off");
+		    if (mute)
+			audio_off();
+		    drv->close(h_drv);
+		    if (fb_switch_state == FB_ACTIVE)
+			fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
+		    tty_cleanup();
+		    fb_cleanup();
 		    fprintf(stderr,"capturing image failed\n");
 		    exit(1);
 		}
 		if (ch)
 		    buf = ng_convert_frame(ch,NULL,buf);
 		/* blit frame */
+		src = buf->data;
+
 		dst = fb_mem +
 		    dy * fb_fix.line_length +
-		    dx * ((fb_var.bits_per_pixel+7)/8);
+		    dx * ((fb_var.bits_per_pixel + 7)/8);
+
+		linesize = buf->fmt.width * (ng_vfmt_to_depth[fmt.fmtid] / 8);
+
+		/* Prevent writing at the next line */
+		if (linesize > fb_fix.line_length)
+			linesize = fb_fix.line_length;
 		for (ui = 0; ui < buf->fmt.height; ui++) {
-		    memcpy(dst, buf->data + ui*buf->fmt.bytesperline,
-			   buf->fmt.bytesperline);
+		    if ((char *)dst + linesize > (char *)fb_mem + fb_mem_offset + fb_fix.smem_len)
+			    break;
+
+		    memcpy(dst, src, linesize);
+		    src += buf->fmt.bytesperline;
 		    dst += fb_fix.line_length;
 		}
 		ng_release_video_buf(buf);
@@ -820,6 +885,13 @@ main(int argc, char *argv[])
 		tv.tv_usec = 0;
 		rc = select(fdmax,&set,NULL,NULL,&tv);
 	    }
+
+	    /* If space is too short, overlay text at screen */
+	    if ((fb_switch_state == FB_ACTIVE || keep_dma_on) && !quiet && dy < f->height) {
+		text_out(0,0,text1);
+		text_out(fb_var.xres - text_width(text2) - f->width, 0, text2);
+	    }
+
 	    err = errno;
 	    if (switch_last != fb_switch_state)
 		console_switch();
