@@ -9,6 +9,13 @@
  */
 #include "config.h"
 
+#ifdef MAJOR_IN_SYSMACROS
+# include <sys/sysmacros.h>
+#endif
+#ifdef MAJOR_IN_MKDEV
+# include <sys/mkdev.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,11 +38,11 @@
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # ifdef HAVE_LIBXXF86DGA
-#  include <X11/extensions/xf86dga.h>
+#  include <X11/extensions/Xxf86dga.h>
 # endif
 #endif
 
-#include "videodev.h"
+#include <linux/types.h>
 #include "videodev2.h"
 
 struct DISPLAYINFO {
@@ -49,10 +56,9 @@ struct DISPLAYINFO {
 
 int    verbose    = 1;
 int    yuv        = 0;
-int    v4l1       = 1;
-int    v4l2       = 1;
 int    user_bpp   = 0;
 int    user_shift = 0;
+int    user_bpl   = 0;
 void   *user_base = NULL;
 char   *display   = NULL;
 char   *fbdev     = NULL;
@@ -97,10 +103,10 @@ displayinfo_mklinux(struct DISPLAYINFO *d)
 {
     struct vc_mode mode;
     int fd;
-	
+
     if (verbose)
 	fprintf(stderr,"v4l-conf: using mklinux console driver\n");
-    
+
     if (-1 == (fd = open("/dev/console",O_RDWR|O_NDELAY))) {
 	fprintf(stderr,"open console: %s\n",strerror(errno));
 	exit(1);
@@ -131,8 +137,8 @@ dev_open(const char *device, int major)
     int	fd;
 
     if (strncmp(device, "/dev/", 5)) {
-        fprintf(stderr, "error: %s is not a /dev file\n", device);
-        exit(1);
+	fprintf(stderr, "error: %s is not a /dev file\n", device);
+	exit(1);
     }
 
     /* open & check v4l device */
@@ -183,13 +189,13 @@ displayinfo_x11(Display *dpy, struct DISPLAYINFO *d)
 
     if (verbose)
 	fprintf(stderr,"v4l-conf: using X11 display %s\n",display);
-    
+
     /* take size from root window */
     root = DefaultRootWindow(dpy);
     XGetWindowAttributes(dpy, root, &wts);
     d->width  = wts.width;
     d->height = wts.height;
-    
+
     /* look for a usable visual */
     template.screen = XDefaultScreen(dpy);
     info = XGetVisualInfo(dpy, VisualScreenMask,&template,&found);
@@ -221,6 +227,21 @@ displayinfo_x11(Display *dpy, struct DISPLAYINFO *d)
     }
 }
 
+#ifdef HAVE_LIBXXF86DGA
+static int dga_error = 0;
+static int dga_error_base;
+static int (*orig_xfree_error_handler)(Display *, XErrorEvent *);
+
+static int dga_error_handler(Display *d, XErrorEvent *e)
+{
+  if (e->error_code == (dga_error_base + XF86DGANoDirectVideoMode)) {
+    dga_error = 1;
+    return 0;
+  }
+  return orig_xfree_error_handler(d, e);
+}
+#endif
+
 static void
 displayinfo_dga(Display *dpy, struct DISPLAYINFO *d)
 {
@@ -228,7 +249,7 @@ displayinfo_dga(Display *dpy, struct DISPLAYINFO *d)
     int                      width,bar,foo,major,minor,flags=0;
     void                     *base = NULL;
 
-    if (!XF86DGAQueryExtension(dpy,&foo,&bar)) {
+    if (!XF86DGAQueryExtension(dpy,&foo,&dga_error_base)) {
 	fprintf(stderr,"WARNING: Your X-Server has no DGA support.\n");
 	return;
     }
@@ -240,7 +261,15 @@ displayinfo_dga(Display *dpy, struct DISPLAYINFO *d)
 	fprintf(stderr,"WARNING: No DGA support available for this display.\n");
 	return;
     }
-    XF86DGAGetVideoLL(dpy,XDefaultScreen(dpy),(void*)&base,&width,&foo,&bar);
+    orig_xfree_error_handler = XSetErrorHandler(dga_error_handler);
+    if (!XF86DGAGetVideoLL(dpy,XDefaultScreen(dpy),(void*)&base,&width,&foo,&bar))
+	dga_error = 1;
+    XSync(dpy, 0);
+    XSetErrorHandler(orig_xfree_error_handler);
+    if (dga_error) {
+	fprintf(stderr,"WARNING: No DGA direct video mode for this display.\n");
+	return;
+    }
     d->bpl  = width * d->bpp/8;
     d->base = base;
 #else
@@ -264,8 +293,8 @@ displayinfo_fbdev(struct DISPLAYINFO *d)
 	    exit(1);
 	}
 	if (-1 == ioctl(fd, VT_GETSTATE, &vstat)) {
-	    perror("ioctl VT_GETSTATE");
-	    exit(1);
+	    perror("VT_GETSTATE is not supported");
+	    return;
 	}
 	close(fd);
 	c2m.console = vstat.v_active;
@@ -283,7 +312,7 @@ displayinfo_fbdev(struct DISPLAYINFO *d)
     }
     if (verbose)
 	fprintf(stderr,"v4l-conf: using framebuffer device %s\n",fbdev);
-    
+
     /* Open frame buffer device, with security checks */
     fd = dev_open(fbdev, 29 /* VIDEO_MAJOR */);
     if (-1 == ioctl(fd,FBIOGET_FSCREENINFO,&fix)) {
@@ -320,12 +349,6 @@ displayinfo_v4l2(int fd, struct DISPLAYINFO *d)
     struct v4l2_capability	cap;
     struct v4l2_framebuffer     fb;
 
-    if (0 == v4l2) {
-	if (verbose)
-	    fprintf(stderr,"skipping v4l2 (disabled on the cmd line)\n");
-	return -1;
-    }
-
     if (-1 == ioctl(fd,VIDIOC_QUERYCAP,&cap)) {
 	if (verbose)
 	    fprintf(stderr,"%s [v4l2]: ioctl VIDIOC_QUERYCAP: %s\n",
@@ -363,7 +386,15 @@ displayinfo_v4l2(int fd, struct DISPLAYINFO *d)
     }
     if (yuv)
 	fb.fmt.pixelformat = V4L2_PIX_FMT_YUYV;
+    /* Prefer an already configured bpl (if it makes sense) over our found bpl
+       if we did not find a base as our bpl is not very reliable when we did
+       not find a base */
+    if (user_bpl || d->base ||
+	    fb.fmt.bytesperline < (fb.fmt.width * ((d->bpp + 7) / 8)))
     fb.fmt.bytesperline = d->bpl;
+    else
+	fprintf(stderr,"WARNING: keeping fbuf pitch at: %d, as no base addr was detected\n",
+	    (int)fb.fmt.bytesperline);
     fb.fmt.sizeimage = fb.fmt.height * fb.fmt.bytesperline;
     if (NULL != d->base)
 	fb.base   = d->base;
@@ -386,65 +417,6 @@ displayinfo_v4l2(int fd, struct DISPLAYINFO *d)
     return 0;
 }
 
-static int
-displayinfo_v4l(int fd, struct DISPLAYINFO *d)
-{
-    struct video_capability  capability;
-    struct video_buffer      fbuf;
-
-    if (0 == v4l1) {
-	if (verbose)
-	    fprintf(stderr,"skipping v4l (disabled on the cmd line)\n");
-	return -1;
-    }
-
-    if (-1 == ioctl(fd,VIDIOCGCAP,&capability)) {
-	fprintf(stderr,"%s [v4l]: ioctl VIDIOCGCAP: %s\n",
-		videodev,strerror(errno));
-	return -1;
-    }
-    if (!(capability.type & VID_TYPE_OVERLAY)) {
-	fprintf(stderr,"%s [v4l]: no overlay support\n",videodev);
-	exit(1);
-    }
-
-    /* read-modify-write v4l screen parameters */
-    if (-1 == ioctl(fd,VIDIOCGFBUF,&fbuf)) {
-	fprintf(stderr,"%s [v4l]: ioctl VIDIOCGFBUF: %s\n",
-		videodev,strerror(errno));
-	exit(1);
-    }
-
-    /* set values */
-    fbuf.width        = d->width;
-    fbuf.height       = d->height;
-    fbuf.depth        = d->bpp;
-    fbuf.bytesperline = d->bpl;
-    if (NULL != d->base)
-	fbuf.base     = d->base;
-    if (NULL == fbuf.base)
-	fprintf(stderr,
-		"WARNING: couldn't find framebuffer base address, try manual\n"
-		"         configuration (\"v4l-conf -a <addr>\")\n");
-
-    /* XXX bttv confuses color depth and bits/pixel */
-    if (d->depth == 15)
-	fbuf.depth = 15;
-
-    if (-1 == ioctl(fd,VIDIOCSFBUF,&fbuf)) {
-	fprintf(stderr,"%s [v4l]: ioctl VIDIOCSFBUF: %s\n",
-		videodev,strerror(errno));
-	if (EPERM == errno  &&  0 != geteuid())
-	    fprintf(stderr,
-		    "v4l-conf: You should install me suid root, I need\n"
-		    "          root priviliges for the VIDIOCSFBUF ioctl.\n");
-	exit(1);
-    }
-    if (verbose)
-	fprintf(stderr,"%s [v4l]: configuration done\n",videodev);
-    return 0;
-}
-
 /* ---------------------------------------------------------------- */
 
 int
@@ -473,10 +445,10 @@ main(int argc, char *argv[])
 	display = h;
     if (NULL != (h = getenv("FRAMEBUFFER")))
 	fbdev = h;
-    
+
     /* parse options */
     for (;;) {
-	if (-1 == (c = getopt(argc, argv, "hyq12d:c:b:s:fa:")))
+	if (-1 == (c = getopt(argc, argv, "hyq12d:c:b:s:fa:p:")))
 	    break;
 	switch (c) {
 	case 'q':
@@ -485,14 +457,6 @@ main(int argc, char *argv[])
 	case 'y':
 	    yuv = 1;
 	    break;
-	case '1':
-	    v4l1 = 1;
-	    v4l2 = 0;
-	    break;
-	case '2':
-	    v4l1 = 0;
-	    v4l2 = 1;
-	    break;
 	case 'd':
 	    display = optarg;
 	    break;
@@ -521,6 +485,14 @@ main(int argc, char *argv[])
 		exit(1);
 	    }
 	    break;
+	case 'p':
+	    if (0 == getuid()) {
+		sscanf(optarg,"%d",&user_bpl);
+	    } else {
+		fprintf(stderr,"only root is allowed to use the -p option\n");
+		exit(1);
+	    }
+	    break;
 	case 'h':
 	default:
 	    fprintf(stderr,
@@ -538,6 +510,8 @@ main(int argc, char *argv[])
 		    "    -a <addr> set framebuffer address to <addr>\n"
 		    "              (in hex, root only, successful autodetect\n"
 		    "               will overwrite this address)\n"
+		    "    -p <pitch> set framebuffer pitch to <pitch> bytes\n"
+		    "              (decimal, root only)\n"
 		    "    -1        force v4l API\n"
 		    "    -2        force v4l2 API\n",
 		    argv[0],
@@ -604,6 +578,9 @@ main(int argc, char *argv[])
 	(d.depth  == 15 || d.depth  == 16))
 	d.depth = user_bpp;
 
+    if (user_bpl)
+	d.bpl = user_bpl;
+
     if (verbose) {
 	fprintf(stderr,"mode: %dx%d, depth=%d, bpp=%d, bpl=%d, ",
 		d.width,d.height,d.depth,d.bpp,d.bpl);
@@ -616,8 +593,7 @@ main(int argc, char *argv[])
     /* Set the parameters (needs root) */
     root_user();
     fd = dev_open(videodev, 81 /* VIDEO_MAJOR */);
-    if (-1 == displayinfo_v4l2(fd,&d))
-	displayinfo_v4l(fd,&d);
+    displayinfo_v4l2(fd,&d);
     close(fd);
     return 0;
 }
