@@ -26,6 +26,7 @@
 # define RTLD_NOW RTLD_LAZY
 #endif
 
+#include "get_media_devices.h"
 #include "grab-ng.h"
 
 int  ng_debug          = 0;
@@ -121,7 +122,7 @@ const char* ng_attr_to_desc[] = {
 void ng_init_video_buf(struct ng_video_buf *buf)
 {
     memset(buf,0,sizeof(*buf));
-    pthread_mutex_init(&buf->lock,NULL);    
+    pthread_mutex_init(&buf->lock,NULL);
     pthread_cond_init(&buf->cond,NULL);
 }
 
@@ -226,7 +227,7 @@ const char*
 ng_attr_getstr(struct ng_attribute *attr, int value)
 {
     int i;
-    
+
     if (NULL == attr)
 	return NULL;
     if (attr->type != ATTR_TYPE_CHOICE)
@@ -242,7 +243,7 @@ int
 ng_attr_getint(struct ng_attribute *attr, char *value)
 {
     int i,val;
-    
+
     if (NULL == attr)
 	return -1;
     if (attr->type != ATTR_TYPE_CHOICE)
@@ -260,7 +261,7 @@ ng_attr_getint(struct ng_attribute *attr, char *value)
 	for (i = 0; attr->choices[i].str != NULL; i++)
 	    if (val == attr->choices[i].nr)
 		return attr->choices[i].nr;
-	
+
     }
     return -1;
 }
@@ -269,7 +270,7 @@ void
 ng_attr_listchoices(struct ng_attribute *attr)
 {
     int i;
-    
+
     fprintf(stderr,"valid choices for \"%s\": ",attr->name);
     for (i = 0; attr->choices[i].str != NULL; i++)
 	fprintf(stderr,"%s\"%s\"",
@@ -409,7 +410,7 @@ ng_aconv_register(int magic, char *plugname,
 		  struct ng_audio_conv *list, int count)
 {
     int n;
-    
+
     if (0 != ng_check_magic(magic,plugname,"audio converters"))
 	return -1;
     for (n = 0; n < count; n++)
@@ -504,7 +505,7 @@ ng_conv_find_from(unsigned int in, int *i)
 {
     struct list_head *item;
     struct ng_video_conv *ret;
-    
+
     int j = 0;
 
     list_for_each(item,&ng_conv) {
@@ -531,7 +532,7 @@ ng_conv_find_match(unsigned int in, unsigned int out)
 {
     struct list_head *item;
     struct ng_video_conv *ret = NULL;
-    
+
     list_for_each(item,&ng_conv) {
 	ret = list_entry(item, struct ng_video_conv, list);
 	if (ret->fmtid_in  == in && ret->fmtid_out == out)
@@ -542,15 +543,142 @@ ng_conv_find_match(unsigned int in, unsigned int out)
 
 /* --------------------------------------------------------------------- */
 
+#ifdef __linux__ /* Because this depends on get_media_devices.c */
+static void *ng_vid_open_auto(struct ng_vid_driver *drv, char *devpath,
+			      int allow_grabber)
+{
+    void *md, *handle = NULL;
+    const char *device = NULL;
+    const char *scan_type = "an analog TV";
+
+    *devpath = 0;
+    md = discover_media_devices();
+    if (!md)
+	goto error;
+
+    /* Step 1: try TV cards first */
+    while (1) {
+	device = get_associated_device(md, device, MEDIA_V4L_VIDEO, NULL, NONE);
+	if (!device)
+	    break; /* No more video devices to try */
+	snprintf(devpath, PATH_MAX, "/dev/%s", device);
+	if (ng_debug)
+	    fprintf(stderr,"vid-open-auto: trying: %s... \n", devpath);
+	handle = (drv->open)(devpath, CAN_CAPTURE | CAN_TUNE);
+	if (handle) {
+		fprintf(stderr,"vid-open-auto: using analog TV device %s\n", devpath);
+		break;
+	}
+    }
+
+    /* Step 2: try grabber devices and webcams */
+    if (!handle) {
+	if (!allow_grabber)
+	    goto error;
+	scan_type = "a capture";
+	device = NULL;
+	while (1) {
+	    device = get_associated_device(md, device, MEDIA_V4L_VIDEO, NULL, NONE);
+	    if (!device)
+		break; /* No more video devices to try */
+	    snprintf(devpath, PATH_MAX, "/dev/%s", device);
+	    if (ng_debug)
+		fprintf(stderr,"vid-open-auto: trying: %s... \n", devpath);
+	    handle = (drv->open)(devpath, CAN_CAPTURE);
+	    if (handle) {
+		fprintf(stderr,"vid-open-auto: using grabber/webcam device %s\n", devpath);
+		break;
+	    }
+	}
+    }
+    free_media_devices(md);
+
+error:
+    if (!handle) {
+	    fprintf(stderr, "vid-open-auto: failed to open %s device",
+		    scan_type);
+	    if (*devpath)
+		    fprintf(stderr, " at %s\n", devpath);
+	    else
+		    fprintf(stderr, "\n");
+
+	    return NULL;
+    }
+
+    if (handle && ng_debug)
+	fprintf(stderr,"vid-open-auto: success, using: %s\n", devpath);
+
+    return handle;
+}
+#endif
+
 const struct ng_vid_driver*
-ng_vid_open(char *device, char *driver, struct ng_video_fmt *screen,
+ng_vid_open(char **device, char *driver, struct ng_video_fmt *screen,
 	    void *base, void **handle)
 {
     struct list_head *item;
     struct ng_vid_driver *drv;
 
+    if (!driver) {
+	fprintf (stderr, "Video4linux driver is not specified\n");
+	return NULL;
+    }
+
+    /* check all grabber drivers */
+    list_for_each(item,&ng_vid_drivers) {
+	drv = list_entry(item, struct ng_vid_driver, list);
+	if (strcasecmp(driver, drv->name) == 0)
+	    break;
+    }
+
+    if (item == &ng_vid_drivers) {
+	if (strcasecmp(driver, "help") != 0)
+	    fprintf (stderr, "Cannot find %s video driver\n", driver);
+	fprintf (stderr, "Available drivers:");
+	list_for_each(item,&ng_vid_drivers) {
+	    drv = list_entry(item, struct ng_vid_driver, list);
+	    fprintf (stderr, " %s", drv->name);
+	}
+	fprintf (stderr, "\n");
+
+	return NULL;
+    }
+
+#ifndef __linux__
+    if (!strcmp(*device, "auto") || !strcmp(*device, "auto_tv"))
+	*device = "/dev/bktr0";
+#else
+    if (!strcmp(*device, "auto") || !strcmp(*device, "auto_tv")) {
+	char devpath[PATH_MAX];
+	*handle = ng_vid_open_auto(drv, devpath,
+				   !strcmp(*device, "auto_tv") ? 0 : 1);
+	if (*handle == NULL) {
+	    fprintf(stderr, "vid-open: could not find a suitable videodev\n");
+	    return NULL;
+	}
+	*device = strdup(devpath);
+    } else
+#endif
+    {
+	if (ng_debug)
+	    fprintf(stderr,"vid-open: trying: %s... \n", drv->name);
+	if (!(*handle = (drv->open)(*device, 0))) {
+	    fprintf(stderr,"vid-open: failed: %s\n", drv->name);
+	    return NULL;
+	}
+	if (ng_debug)
+	    fprintf(stderr,"vid-open: ok: %s\n", drv->name);
+    }
+
+    if (NULL != screen && drv->capabilities(*handle) & CAN_OVERLAY) {
 #ifdef __linux__
-    if (NULL != screen) {
+	int l = strlen(ng_v4l_conf);
+
+	snprintf(ng_v4l_conf + l, sizeof(ng_v4l_conf) - l, " -c %s", *device);
+
+	if (ng_debug)
+	    fprintf(stderr,"vid-open: closing dev to run v4lconf\n");
+	drv->close(*handle);
 	switch (system(ng_v4l_conf)) {
 	case -1: /* can't run */
 	    fprintf(stderr,"could'nt start v4l-conf\n");
@@ -562,27 +690,18 @@ ng_vid_open(char *device, char *driver, struct ng_video_fmt *screen,
 		    "trying to continue anyway\n");
 	    break;
 	}
-    }
+	if (ng_debug)
+	    fprintf(stderr,"vid-open: re-opening dev after v4lconf\n");
+	if (!(*handle = (drv->open)(*device, 0))) {
+	    fprintf(stderr,"vid-open: failed: %s\n", drv->name);
+	    return NULL;
+	}
+	if (ng_debug)
+	    fprintf(stderr,"vid-open: re-open ok\n");
 #endif
-
-    /* check all grabber drivers */
-    list_for_each(item,&ng_vid_drivers) {
-        drv = list_entry(item, struct ng_vid_driver, list);
-	if (driver && 0 != strcasecmp(driver, drv->name))
-	    continue;
-	if (ng_debug)
-	    fprintf(stderr,"vid-open: trying: %s... \n", drv->name);
-	if (NULL != (*handle = drv->open(device)))
-	    break;
-	if (ng_debug)
-	    fprintf(stderr,"vid-open: failed: %s\n",drv->name);
-    }
-    if (item == &ng_vid_drivers)
-	return NULL;
-    if (ng_debug)
-	fprintf(stderr,"vid-open: ok: %s\n",drv->name);
-    if (NULL != screen && drv->capabilities(*handle) & CAN_OVERLAY)
 	drv->setupfb(*handle,screen,base);
+    }
+
     return drv;
 }
 
@@ -594,7 +713,7 @@ ng_dsp_open(char *device, struct ng_audio_fmt *fmt, int record, void **handle)
 
     /* check all dsp drivers */
     list_for_each(item,&ng_dsp_drivers) {
-        drv = list_entry(item, struct ng_dsp_driver, list);
+	drv = list_entry(item, struct ng_dsp_driver, list);
 	if (NULL == drv->name)
 	    continue;
 	if (record && NULL == drv->read)
@@ -603,7 +722,7 @@ ng_dsp_open(char *device, struct ng_audio_fmt *fmt, int record, void **handle)
 	    continue;
 	if (ng_debug)
 	    fprintf(stderr,"dsp-open: trying: %s... \n", drv->name);
-	if (NULL != (*handle = drv->open(device,fmt,record)))
+	if (NULL != (*handle = (drv->open)(device,fmt,record)))
 	    break;
 	if (ng_debug)
 	    fprintf(stderr,"dsp-open: failed: %s\n", drv->name);
@@ -622,13 +741,13 @@ ng_mix_init(char *device, char *channel)
     struct ng_mix_driver *drv = NULL;
     struct ng_attribute *attrs = NULL;
     void *handle;
-    
+
     /* check all mixer drivers */
     list_for_each(item, &ng_mix_drivers) {
-        drv = list_entry(item, struct ng_mix_driver, list);
+	drv = list_entry(item, struct ng_mix_driver, list);
 	if (ng_debug)
 	    fprintf(stderr,"mix-init: trying: %s... \n", drv->name);
-	if (NULL != (handle = drv->open(device))) {
+	if (NULL != (handle = (drv->open)(device))) {
 	    if (NULL != (attrs = drv->volctl(handle,channel)))
 		break;
 	    drv->close(handle);
@@ -651,7 +770,7 @@ struct ng_reader* ng_find_reader(char *filename)
 
     if (NULL == (fp = fopen(filename, "r"))) {
 	fprintf(stderr,"open %s: %s\n",filename,strerror(errno));
-        return NULL;
+	return NULL;
     }
     memset(blk,0,sizeof(blk));
     fread(blk,1,sizeof(blk),fp);
